*******************************************************************************
*******************************************************************************
** Name: MGRID
** Machine: SX/GX
** Language: sys-rpl, ml
** Interface: %size %rows %cols ->
** Abstract: Draws in the display a centered grid. %size specifies the size
** of each square without counting box boundaries.
** Author: 	Daniel Lidstrom
**		Nackstav. 22C
**		853 52  Sundsvall
**		Sweden
**		email: danli97@ite.mh.se
**		www: http://www.ite.mh.se/~danli97/
** Technical:
** History:	v 1.0	1998-12-04	first release	DL
**
*******************************************************************************
*******************************************************************************


ASSEMBLE
UnScroll	EQU	#13695 U
RPL
::
 0LASTOWDOB!
 CK3NOLASTWD
 CK&DISPATCH1
 3REAL
 ::
  COERCE2 ROT COERCE #1+
  DUP ROT #* #1+ DUP BINT_131d
  #> case SETSIZEERR
  OVER 4ROLL #* #1+
  DUP 63 #> case SETSIZEERR
  ClrDA1IsStat TOADISP
  UnScroll TURNMENUOFF
  CLEARVDISP
  SIXTYTHREE OVER #-#2/
  DUP ROT #+-1 BINT_131d
  4PICK #-#2/ DUP 5ROLL #+-1
( #size+1 #toprow #botrow #lfcol #rtcol )
CODE
sSHORT	EQU	1		flag indicating row fits inside single nibble
	GOSBVL	=SAVPTR
	GOSBVL	=D0->Row1	D0->top left row
	GOSBVL	=POP2#		
	R1=A.F	A		left x
	C=C-A	A
	C=C+1	A
	R2=C.F	A		no. of pixels/row
	GOSBVL	=POP2#
	R3=A.F	A		top y
	C=C-A	A
	R4=C.F	A		column length
	GOSBVL	=POP#
	R0=A.F	A		size+1
	C=R4.F	A
	ACEX	A
	GOSBVL	=IntDiv		C[A]=no. of horizontal lines
	RSTK=C			keep counter here
	CD0EX
	A=R3.F	A		y
	A=A+A	A		2*y
	B=A	A
	ASL	A		32*y
	A=A+B	A		34*y
	C=C+A	A
	A=R1.F	A
	ASRB.F	A
	ASRB.F	A		x >> 4
	C=C+A	A
	D0=C			D0->top left nibble of grid

* compute D0 new line add amount
* this will be added to D0 when it is time to draw a new line
	C=0	A
	A=0	A
	LC(2)	34
	A=R0.F	B
	GOSBVL	=MUL#		B[A]=34*(size+1)
	B=B-1	A		left mask compensation

* compute masks
* left mask
	ST=0	sSHORT		assume larger grid
	C=R2.F	A
	D=C	A
	D=D-CON	A,4		assume mask is 4 pixels
	GOC	rtX		if smaller, only right mask
	LCHEX	F		4 pixels
* shift mask
	A=R1.F	B
	?ABIT=0	0
	GOYES	+
	C=C+C	P
	D=D+1	A
+	?ABIT=0	1
	GOYES	+
	C=C+C	P
	C=C+C	P
	D=D+1	A
	D=D+1	A
+	CSRC
	B=C	S		B[S]=left mask
* compute amount to compensate for middle part of row in D0 add amount
	C=D	A
	CSRB.F	A
	CSRB.F	A		x >> 4
	C=C+1	A		compensate for right mask
	B=B-C	A		subtract
	?D=0	A		any more pixels?
	GOYES	+		no, skip this mask
* right mask
rtX	D=D+CON	A,4		if D[A]<0, we have a small grid
	GONC	+
	ST=1	sSHORT		flag small grid
	D=D+CON	A,4		fix D[A] for next instruction
+	D=D-CON	A,8		D[A]-=4
	A=R1.F	A
	C=R2.F	A
	A=A+C	A
	A=A-1	A		A[A]=right x coordinate of grid
* shift mask
	LCHEX	F
	?ABIT=1	0
	GOYES	+
	CSRB.F	P
	D=D+1	A
+	?ABIT=1	1
	GOYES	+
	CSRB.F	P
	CSRB.F	P
	D=D+1	A
	D=D+1	A
+	CSRC
	D=C	S		D[S]=right mask
	C=C+1	S		if mask == F we must adjust B[A]
	GONC	+
	B=B+1	A		adjust D0 add amount

* masks computed
* start drawing
+	A=0	W
	A=A-1	W		64 pixels
	DSRB.F	A
	DSRB.F	A		4 pixels/nibble
--	?ST=1	sSHORT		small grid?
	GOYES	lastX
	C=B	S
	DAT0=C	S		left mask
	D0=D0+	1

* compute counters

	C=D	A
	P=C	0		P=nibbles mod 16
	CSR	A		C[A]=nibbles/16
	C=C-1	A		trap 0 case
	GOC	+
-	DAT0=A	W		write
	D0=D0+	16
	C=C-1	A
	GONC	-
+	P=P-1			trap 0 case
	GOC	doneX
	DAT0=A	WP		write last nibbles
doneX	CD0EX
	?P=	15
	GOYES	setD0
	C+P+1			advance D0
setD0	D0=C
	P=	0
lastX	C=D	S
	DAT0=C	S		write right mask
	D0=D0+	1
	CD0EX
	C=C+B	A		add D0 add amount
	D0=C			D0 -> next row
	C=RSTK
	C=C-1	A
	RSTK=C
	GONC	--		done?
*******************************************************************************
* time to draw vertical lines ( a lot easier! )
*******************************************************************************
	C=R4.F	A
	C=C-1	A
	C=C-1	A
	R4=C.F	A		no need to draw top y and bottom y
	A=R2.F	A
	A=A-1	A
	C=R0.F	A
	GOSBVL	=IntDiv
	RSTK=C			RSTK=lines
	GOSBVL	=D0->Row1
	C=R3.F	A
	C=C+1	A		y
	C=C+C	A		2*y
	B=C	A
	CSL	A		32*y
	C=C+B	A		34*y
	C=C+A	A
	R2=C.F	A		top left of grid
	A=R1.F	A		left x
	CAEX	A
	D=C	A		D[A]=C[A]=x
* shift bit
--	LA(1)	1
	?CBIT=0	0
	GOYES	+
	A=A+A	A
+	?CBIT=0	1
	GOYES	+
	A=A+A	A
	A=A+A	A
+	CSRB.F	A
	CSRB.F	A		C[A] = x >> 4
	AR2EX.F	A
	C=C+A	A
	AR2EX.F	A
	D0=C			D0 -> current x of line
	C=R4.F	A
	B=C	A		B[A] = y counter
-	C=DAT0	A
	C=C!A	P		or pixel in
	DAT0=C	1		write
	D0=D0+	16
	D0=D0+	16
	D0=D0+	2		D0 down one
	B=B-1	A
	GONC	-
	C=R0.F	A
	D=D+C	A		D[A] = next x
	C=RSTK
	C=C-1	A		done? (*)
	RSTK=C
	C=D	A		set C[A] = x
	GONC	--		loop if no carry in (*)
	GOVLNG	=GETPTRLOOP
ENDCODE
  5DROP
  SetDAsTemp		( freeze display )
 ;
;
